export const TOGGLE_CLASSES = {
    active: 'active',
    show: 'show',
    inactive: ''
};
export class StbCarousel {
    constructor() {
        this.selected = 0;
        this.slideSelector = '.carousel-item';
        this.indicatorSelector = '[data-slide-to]';
        this.prevSelector = '[data-slide="prev"]';
        this.nextSelector = '[data-slide="next"]';
        this.interval = 5000; //default
        this.cycleType = 'hover';
        this.ride = false;
        this.wrap = true;
        this.listeners = {};
        this.currentSlide = 0;
    }
    componentWillLoad() {
        this.currentSlide = this.selected || 0;
        this.getSlideElements();
        this.getIndicatorElements();
        this.getPrevNextButtons();
        if (this.currentSlide > 0) {
            this.show(this.selected);
        }
        this.play();
    }
    componentDidUnload() {
        if (this.slideElements && this.slideElements.length) {
            this.slideElements.forEach((element, i) => {
                element.removeEventListener('click', this.listeners[i]);
            });
        }
        if (this.prevElement && this.prevListener) {
            this.prevElement.removeEventListener('click', this.prevListener);
        }
        if (this.nextElement && this.nextListener) {
            this.nextElement.removeEventListener('click', this.nextListener);
        }
        this.pause();
    }
    getPrevNextButtons() {
        this.prevElement = this.element.querySelector(this.prevSelector);
        this.nextElement = this.element.querySelector(this.nextSelector);
        if (this.prevElement) {
            this.prevListener = this.addClickHandler(this.prevElement, () => this.prev());
        }
        if (this.nextElement) {
            this.nextListener = this.addClickHandler(this.nextElement, () => this.next());
        }
    }
    getSlideElements() {
        this.slideElements = this.element.querySelectorAll(this.slideSelector);
    }
    getIndicatorElements() {
        this.indicatorElements = this.element.querySelectorAll(this.indicatorSelector);
        this.indicatorElements.forEach((element, i) => {
            this.indicatorClickHandler(element, i);
        });
    }
    addClickHandler(element, fn = this.next) {
        if (!element)
            return;
        let eventHandler = () => {
            if (fn) {
                fn();
            }
        };
        element.addEventListener('click', eventHandler);
        return eventHandler;
    }
    indicatorClickHandler(element, index) {
        if (!element)
            return;
        let eventHandler = () => {
            this.show(index);
        };
        element.addEventListener('click', eventHandler);
        this.listeners[index] = eventHandler;
    }
    addActiveClass(element) {
        element.classList.add(TOGGLE_CLASSES.active);
        if (TOGGLE_CLASSES.inactive) {
            element.classList.remove(TOGGLE_CLASSES.inactive);
        }
    }
    removeActiveClass(elements = this.slideElements) {
        elements.forEach(item => {
            item.classList.remove(TOGGLE_CLASSES.active);
            if (TOGGLE_CLASSES.inactive) {
                item.classList.add(TOGGLE_CLASSES.inactive);
            }
        });
    }
    show(index = 0) {
        this.removeActiveClass();
        if (this.indicatorElements && this.indicatorElements.length) {
            this.removeActiveClass(this.indicatorElements);
        }
        this.indicatorClickHandler(this.slideElements[index], index);
        this.addActiveClass(this.slideElements[index]);
        this.currentSlide = index;
        this.pause();
        this.play();
        this.onShow.emit({
            current: this.currentSlide,
            element: this.slideElements[index]
        });
    }
    hide(index = 0) {
        this.removeActiveClass();
        if (this.indicatorElements && this.indicatorElements.length) {
            this.removeActiveClass(this.indicatorElements);
        }
        this.onHide.emit({
            current: this.currentSlide,
            element: this.slideElements[index]
        });
        this.currentSlide = 0;
    }
    next() {
        if (this.slideElements && this.slideElements.length - 1 <= this.currentSlide) {
            this.show(0);
        }
        else {
            this.show(this.currentSlide + 1);
        }
    }
    prev() {
        if (this.currentSlide <= 0) {
            this.show(this.slideElements.length - 1);
        }
        else {
            this.show(this.currentSlide - 1);
        }
    }
    pause() {
        if (this.intervalHandler) {
            clearInterval(this.intervalHandler);
        }
    }
    play() {
        if (this.interval) {
            this.intervalHandler = setInterval(() => this.next(), this.interval);
        }
    }
    static get is() { return "stb-carousel"; }
    static get host() { return { "theme": "" }; }
    static get properties() { return { "cycleType": { "type": "Any", "attr": "cycle-type" }, "element": { "elementRef": true }, "hide": { "method": true }, "indicatorSelector": { "type": String, "attr": "indicator-selector" }, "interval": { "type": Number, "attr": "interval" }, "next": { "method": true }, "nextSelector": { "type": String, "attr": "next-selector" }, "pause": { "method": true }, "play": { "method": true }, "prev": { "method": true }, "prevSelector": { "type": String, "attr": "prev-selector" }, "ride": { "type": Boolean, "attr": "ride" }, "selected": { "type": Number, "attr": "selected" }, "show": { "method": true }, "slideSelector": { "type": String, "attr": "slide-selector" }, "wrap": { "type": Boolean, "attr": "wrap" } }; }
    static get events() { return [{ "name": "onShow", "method": "onShow", "bubbles": true, "cancelable": true, "composed": true }, { "name": "onHide", "method": "onHide", "bubbles": true, "cancelable": true, "composed": true }]; }
}
